@@ -39,7 +39,7 @@ static void store_printer_guid(struct messaging_context *msg_ctx,
 			       const char *printer, struct GUID guid)
 {
 	TALLOC_CTX *tmp_ctx;
-	struct auth_serversupplied_info *session_info = NULL;
+	struct auth_session_info *session_info = NULL;
 	const char *guid_str;
 	DATA_BLOB blob;
 	NTSTATUS status;
@@ -94,7 +94,7 @@ static WERROR nt_printer_info_to_mods(TALLOC_CTX *ctx,
 	char *info_str;
 
 	ads_mod_str(ctx, mods, SPOOL_REG_PRINTERNAME, info2->sharename);
-	ads_mod_str(ctx, mods, SPOOL_REG_SHORTSERVERNAME, global_myname());
+	ads_mod_str(ctx, mods, SPOOL_REG_SHORTSERVERNAME, lp_netbios_name());
 	ads_mod_str(ctx, mods, SPOOL_REG_SERVERNAME, get_mydnsfullname());
 
 	info_str = talloc_asprintf(ctx, "\\\\%s\\%s",
@@ -192,10 +192,10 @@ static WERROR nt_printer_publish_ads(struct messaging_context *msg_ctx,
 	DEBUG(5, ("publishing printer %s\n", printer));
 
 	/* figure out where to publish */
-	ads_rc = ads_find_machine_acct(ads, &res, global_myname());
+	ads_rc = ads_find_machine_acct(ads, &res, lp_netbios_name());
 	if (!ADS_ERR_OK(ads_rc)) {
 		DEBUG(0, ("failed to find machine account for %s\n",
-			  global_myname()));
+			  lp_netbios_name()));
 		TALLOC_FREE(ctx);
 		return WERR_NOT_FOUND;
 	}
@@ -304,7 +304,7 @@ static WERROR nt_printer_unpublish_ads(ADS_STRUCT *ads,
 
 	/* remove the printer from the directory */
 	ads_rc = ads_find_printer_on_server(ads, &res,
-					    printer, global_myname());
+					    printer, lp_netbios_name());
 
 	if (ADS_ERR_OK(ads_rc) && res && ads_count_replies(ads, res)) {
 		prt_dn = ads_get_dn(ads, talloc_tos(), res);
@@ -333,7 +333,7 @@ static WERROR nt_printer_unpublish_ads(ADS_STRUCT *ads,
  ***************************************************************************/
 
 WERROR nt_printer_publish(TALLOC_CTX *mem_ctx,
-			  const struct auth_serversupplied_info *session_info,
+			  const struct auth_session_info *session_info,
 			  struct messaging_context *msg_ctx,
 			  struct spoolss_PrinterInfo2 *pinfo2,
 			  int action)
@@ -415,7 +415,7 @@ WERROR check_published_printers(struct messaging_context *msg_ctx)
 	int snum;
 	int n_services = lp_numservices();
 	TALLOC_CTX *tmp_ctx = NULL;
-	struct auth_serversupplied_info *session_info = NULL;
+	struct auth_session_info *session_info = NULL;
 	struct spoolss_PrinterInfo2 *pinfo2;
 	NTSTATUS status;
 	WERROR result;
@@ -455,7 +455,7 @@ WERROR check_published_printers(struct messaging_context *msg_ctx)
 		}
 
 		result = winreg_get_printer_internal(tmp_ctx, session_info, msg_ctx,
-					    lp_servicename(snum),
+					    lp_servicename(talloc_tos(), snum),
 					    &pinfo2);
 		if (!W_ERROR_IS_OK(result)) {
 			continue;
@@ -477,9 +477,11 @@ done:
 }
 
 bool is_printer_published(TALLOC_CTX *mem_ctx,
-			  const struct auth_serversupplied_info *session_info,
+			  const struct auth_session_info *session_info,
 			  struct messaging_context *msg_ctx,
-			  const char *servername, char *printer, struct GUID *guid,
+			  const char *servername,
+			  const char *printer,
+			  struct GUID *guid,
 			  struct spoolss_PrinterInfo2 **info2)
 {
 	struct spoolss_PrinterInfo2 *pinfo2 = NULL;
@@ -558,7 +560,7 @@ done:
 }
 #else
 WERROR nt_printer_publish(TALLOC_CTX *mem_ctx,
-			  const struct auth_serversupplied_info *session_info,
+			  const struct auth_session_info *session_info,
 			  struct messaging_context *msg_ctx,
 			  struct spoolss_PrinterInfo2 *pinfo2,
 			  int action)
@@ -572,9 +574,11 @@ WERROR check_published_printers(struct messaging_context *msg_ctx)
 }
 
 bool is_printer_published(TALLOC_CTX *mem_ctx,
-			  const struct auth_serversupplied_info *session_info,
+			  const struct auth_session_info *session_info,
 			  struct messaging_context *msg_ctx,
-			  const char *servername, char *printer, struct GUID *guid,
+			  const char *servername,
+			  const char *printer,
+			  struct GUID *guid,
 			  struct spoolss_PrinterInfo2 **info2)
 {
 	return False;
